"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SplitModeView = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../classname");
const bundle_1 = require("../i18n/bundle");
const lodash_1 = require("../lodash");
const sync_scroll_1 = require("../utils/sync-scroll");
const b = (0, classname_1.cn)('markup-preview');
const SplitModeView = react_1.default.forwardRef(({ editor }, ref) => {
    var _a;
    const cm = editor.cm;
    const [, forceUpdate] = (0, react_1.useReducer)((x) => x + 1, 0);
    const outerRef = (0, react_1.useRef)(null);
    const lineElements = (0, react_1.useRef)([]);
    const updateLineElements = () => {
        if (!outerRef.current)
            return;
        const els = Array.from(outerRef.current.querySelectorAll('.line'));
        lineElements.current = els
            .filter((el) => { var _a; return el instanceof HTMLElement && ((_a = el.parentElement) === null || _a === void 0 ? void 0 : _a.offsetHeight) !== 0; })
            .map((el) => ({
            el: el,
            line: Number(el.getAttribute('data-line')),
        }));
    };
    const handleEditorScroll = (0, react_1.useMemo)(() => (0, lodash_1.throttle)(() => {
        if (!outerRef.current)
            return;
        if (!cm.dom.matches(':hover'))
            return;
        const { range } = cm.scrollSnapshot().value;
        const line = cm.state.doc.lineAt(range.from).number - 1;
        updateLineElements();
        (0, sync_scroll_1.scrollToRevealSourceLine)(line, lineElements.current, outerRef);
    }, 30), [cm]);
    const handlePreviewScroll = (0, react_1.useMemo)(() => (0, lodash_1.throttle)(() => {
        if (!outerRef.current)
            return;
        if (!outerRef.current.matches(':hover'))
            return;
        updateLineElements();
        let lineNumber = (0, sync_scroll_1.getEditorLineNumberForOffset)(outerRef.current.scrollTop, lineElements.current, outerRef);
        if (lineNumber === null || isNaN(lineNumber))
            return;
        lineNumber = Math.max(Math.floor(lineNumber), 0) + 1;
        const line = cm.state.doc.line(lineNumber);
        const { top } = cm.lineBlockAt(line.from);
        cm.scrollDOM.scrollTo({ top });
    }, 30), [cm]);
    (0, react_1.useEffect)(() => {
        const outer = outerRef.current;
        editor.on('change', forceUpdate);
        editor.on('cm-scroll', handleEditorScroll);
        outer === null || outer === void 0 ? void 0 : outer.addEventListener('scroll', handlePreviewScroll);
        editor.cm.requestMeasure();
        return () => {
            editor.off('change', forceUpdate);
            editor.off('cm-scroll', handleEditorScroll);
            outer === null || outer === void 0 ? void 0 : outer.removeEventListener('scroll', handlePreviewScroll);
        };
    }, [editor, handleEditorScroll, handlePreviewScroll]);
    (0, react_1.useImperativeHandle)(ref, () => outerRef.current);
    return (react_1.default.createElement("div", { className: b('outer', {
            vertical: editor.splitMode === 'vertical',
        }), ref: outerRef },
        react_1.default.createElement(uikit_1.Label, { icon: react_1.default.createElement(icons_1.Eye, null), className: b('preview-sign'), size: 'm' }, (0, bundle_1.i18n)('split-mode-text')), (_a = editor.renderPreview) === null || _a === void 0 ? void 0 :
        _a.call(editor, { getValue: editor.getValue, mode: 'split' })));
});
exports.SplitModeView = SplitModeView;
SplitModeView.displayName = 'SplitModeView';
