"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarButtonWithPopupMenu = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../classname");
const lodash_1 = require("../../lodash");
const hooks_1 = require("../../react-utils/hooks");
const toolbar_1 = require("../../toolbar");
const b = (0, classname_1.cn)('toolbar-button-with-popup-menu');
const ToolbarButtonWithPopupMenu = ({ className, focus, onClick, icon, title, menuItems, }) => {
    const buttonRef = react_1.default.useRef(null);
    const [open, , hide, toggleOpen] = (0, hooks_1.useBooleanState)(false);
    const groups = react_1.default.useMemo(() => (0, lodash_1.groupBy)(menuItems.map((i) => (Object.assign(Object.assign({}, i), { group: i.group || '' }))), 'group'), [menuItems, lodash_1.groupBy]);
    const someActive = menuItems.some((item) => !item.ignoreActive && item.action.isActive() === true);
    const everyDisabled = menuItems.every((item) => item.action.isEnable() === false);
    const popupOpen = everyDisabled ? false : open;
    const shouldForceHide = open && !popupOpen;
    react_1.default.useEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(uikit_1.ActionTooltip, { disabled: popupOpen, openDelay: toolbar_1.ToolbarTooltipDelay.Open, closeDelay: toolbar_1.ToolbarTooltipDelay.Close, title: (0, lodash_1.isFunction)(title) ? title() : title },
            react_1.default.createElement(uikit_1.Button, { size: "m", ref: buttonRef, view: someActive || popupOpen ? 'normal' : 'flat', selected: someActive, disabled: everyDisabled, className: b(null, [className]), onClick: toggleOpen },
                react_1.default.createElement(uikit_1.Icon, { data: icon.data, size: icon.size }),
                '',
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.ChevronDown }))),
        react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide },
            react_1.default.createElement(uikit_1.Menu, { size: "l" }, Object.entries(groups).map(([label, items], key) => {
                return (react_1.default.createElement(uikit_1.Menu.Group, { label: label, key: key, className: b('menu-group') }, items.map(({ id, icon, iconSize = 16, action, text, iconClassname }) => (react_1.default.createElement(uikit_1.Menu.Item, { key: id, icon: react_1.default.createElement(uikit_1.Icon, { data: icon, size: iconSize, className: iconClassname }), active: action.isActive(), disabled: !action.isEnable(), onClick: () => {
                        hide();
                        focus();
                        action.run();
                        onClick === null || onClick === void 0 ? void 0 : onClick(id);
                    }, extraProps: { 'aria-label': text } }, text)))));
            })))));
};
exports.ToolbarButtonWithPopupMenu = ToolbarButtonWithPopupMenu;
