"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MToolbarImagePopup = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const is_number_1 = tslib_1.__importDefault(require("is-number"));
const markup_1 = require("../../../markup");
const commands_1 = require("../../../markup/commands");
const ToolbarImagePopup_1 = require("../custom/ToolbarImagePopup");
const context_1 = require("./context");
const noop = (err) => {
    console.error(err);
};
const MToolbarImagePopup = ({ focus, onClick, hide, anchorRef, editor, className, }) => {
    const { uploadHandler, needToSetDimensionsForUploadedImages } = (0, context_1.useMarkupToolbarContext)();
    return (react_1.default.createElement(ToolbarImagePopup_1.ToolbarImagePopup, { hide: hide, anchorRef: anchorRef, focus: focus, onClick: onClick, className: className, onSubmit: ({ url, name, alt, width, height }) => {
            (0, commands_1.insertImages)([
                {
                    url,
                    alt,
                    title: name,
                    width: (0, is_number_1.default)(width) ? String(width) : '',
                    height: (0, is_number_1.default)(height) ? String(height) : '',
                },
            ])(editor.cm);
        }, uploadImages: uploadHandler, onSuccessUpload: async (res) => {
            const images = await toImageItems(res.success, Boolean(needToSetDimensionsForUploadedImages));
            (0, commands_1.insertImages)(images)(editor.cm);
        } }));
};
exports.MToolbarImagePopup = MToolbarImagePopup;
async function toImageItems(items, withDimmensions) {
    const imgItems = [];
    await Promise.all(items.map(({ result, file }) => {
        var _a;
        const imgItem = { url: result.url, alt: (_a = result.name) !== null && _a !== void 0 ? _a : file.name };
        imgItems.push(imgItem);
        if (withDimmensions) {
            return (0, markup_1.getImageDimensions)(file).then(({ height }) => {
                imgItem.height = String(Math.min(height, markup_1.IMG_MAX_HEIGHT));
            }, noop);
        }
        return Promise.resolve();
    }));
    return imgItems;
}
