"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HorizontalRule = exports.markupAttr = exports.horizontalRule = exports.horizontalRuleType = exports.horizontalRuleNodeName = exports.horizontalRuleMarkupAttr = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const inputrules_1 = require("../../../utils/inputrules");
const selection_1 = require("../../../utils/selection");
const BaseSchemaSpecs_1 = require("../../base/BaseSchema/BaseSchemaSpecs");
const HorizontalRuleSpecs_1 = require("./HorizontalRuleSpecs");
var HorizontalRuleSpecs_2 = require("./HorizontalRuleSpecs");
Object.defineProperty(exports, "horizontalRuleMarkupAttr", { enumerable: true, get: function () { return HorizontalRuleSpecs_2.horizontalRuleMarkupAttr; } });
Object.defineProperty(exports, "horizontalRuleNodeName", { enumerable: true, get: function () { return HorizontalRuleSpecs_2.horizontalRuleNodeName; } });
Object.defineProperty(exports, "horizontalRuleType", { enumerable: true, get: function () { return HorizontalRuleSpecs_2.horizontalRuleType; } });
/** @deprecated Use `horizontalRuleNodeName` instead */
exports.horizontalRule = HorizontalRuleSpecs_1.horizontalRuleNodeName;
const hrAction = 'hRule';
/** @deprecated Use `horizontalRuleMarkupAttr` instead */
exports.markupAttr = HorizontalRuleSpecs_1.horizontalRuleMarkupAttr;
/** @deprecated Use `horizontalRuleType` instead */
const hrType = HorizontalRuleSpecs_1.horizontalRuleType;
const HorizontalRule = (builder) => {
    builder.use(HorizontalRuleSpecs_1.HorizontalRuleSpecs);
    builder.addInputRules((deps) => ({
        rules: [
            // --- or ___ or *** at start of line are converted to horizontal line
            // and add new empty paragraph after
            (0, inputrules_1.nodeInputRule)(/^(---|___|\*\*\*)$/, (markup) => [
                hrType(deps.schema).create({ [HorizontalRuleSpecs_1.horizontalRuleMarkupAttr]: markup }),
                (0, BaseSchemaSpecs_1.pType)(deps.schema).create(),
            ], 1),
        ],
    }));
    builder.addAction(hrAction, ({ schema }) => {
        const cmd = addHr(hrType(schema));
        return {
            isActive: (state) => isHrSelection(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
};
exports.HorizontalRule = HorizontalRule;
const addHr = (hr) => (state, dispatch) => {
    if (isHrSelection(state.selection))
        return true;
    if ((0, selection_1.isNodeSelection)(state.selection)) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(hr.create()).scrollIntoView());
        return true;
    }
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (!$cursor || $cursor.parent.type.spec.complex)
        return false;
    if (!dispatch)
        return true;
    const { tr } = state;
    const isEmptyParagraph = $cursor.parent.type === (0, BaseSchemaSpecs_1.pType)(state.schema) && $cursor.parent.nodeSize === 2;
    if (isEmptyParagraph) {
        const pos = $cursor.before();
        tr.insert(pos, hr.create());
    }
    else {
        const pos = $cursor.after();
        tr.insert(pos, [hr.create(), (0, BaseSchemaSpecs_1.pType)(state.schema).create()]);
        tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, pos + 2)); // set cursor to paragraph after hr
    }
    dispatch(tr.scrollIntoView());
    return true;
};
function isHrSelection(selection) {
    return (0, selection_1.isNodeSelection)(selection) && selection.node.type.name === HorizontalRuleSpecs_1.horizontalRuleNodeName;
}
