"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Checkbox = exports.checkboxInputType = exports.checkboxLabelType = exports.checkboxType = exports.CheckboxNode = void 0;
const prosemirror_utils_1 = require("prosemirror-utils");
const inputrules_1 = require("../../../utils/inputrules");
const BaseSchema_1 = require("../../base/BaseSchema");
const CheckboxSpecs_1 = require("./CheckboxSpecs");
const actions_1 = require("./actions");
const const_1 = require("./const");
const plugin_1 = require("./plugin");
const utils_1 = require("./utils");
const checkboxAction = 'addCheckbox';
var CheckboxSpecs_2 = require("./CheckboxSpecs");
Object.defineProperty(exports, "CheckboxNode", { enumerable: true, get: function () { return CheckboxSpecs_2.CheckboxNode; } });
Object.defineProperty(exports, "checkboxType", { enumerable: true, get: function () { return CheckboxSpecs_2.checkboxType; } });
Object.defineProperty(exports, "checkboxLabelType", { enumerable: true, get: function () { return CheckboxSpecs_2.checkboxLabelType; } });
Object.defineProperty(exports, "checkboxInputType", { enumerable: true, get: function () { return CheckboxSpecs_2.checkboxInputType; } });
const Checkbox = (builder, opts) => {
    builder.use(CheckboxSpecs_1.CheckboxSpecs, Object.assign(Object.assign({}, opts), { inputView: () => (node, view, getPos) => {
            const dom = document.createElement('input');
            for (const attr in node.attrs) {
                if (node.attrs[attr])
                    dom.setAttribute(attr, node.attrs[attr]);
            }
            dom.setAttribute('class', (0, const_1.b)('input'));
            dom.addEventListener('click', (e) => {
                const elem = e.target;
                const checkedAttr = elem.getAttribute('checked');
                const checked = checkedAttr ? '' : 'true';
                const pos = getPos();
                if (pos !== undefined) {
                    view.dispatch(view.state.tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, node.attrs), { checked })));
                }
                elem.setAttribute('checked', checked);
            });
            return {
                dom,
                ignoreMutation: () => true,
                update: () => true,
                destroy() {
                    const pos = getPos();
                    if (pos !== undefined) {
                        const resolved = view.state.doc.resolve(pos);
                        if (resolved.parent.type.name === const_1.CheckboxNode.Checkbox &&
                            resolved.parent.lastChild) {
                            view.dispatch((0, prosemirror_utils_1.replaceParentNodeOfType)(resolved.parent.type, (0, BaseSchema_1.pType)(view.state.schema).create(resolved.parent.lastChild.content))(view.state.tr));
                        }
                    }
                    dom.remove();
                },
            };
        } }));
    builder
        .addPlugin(plugin_1.keymapPlugin, builder.Priority.High)
        .addAction(checkboxAction, () => (0, actions_1.addCheckbox)())
        .addInputRules(({ schema }) => ({
        rules: [
            (0, inputrules_1.nodeInputRule)(/^\[(\s?)\]\s$/, (0, utils_1.checkboxType)(schema).createAndFill({}, (0, utils_1.checkboxInputType)(schema).create()), 2),
        ],
    }));
};
exports.Checkbox = Checkbox;
