"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImagesUploadProcess = void 0;
const logger_1 = require("../../../../logger");
const upload_1 = require("../../../behavior/utils/upload");
const markdown_1 = require("../../../markdown");
const const_1 = require("../const");
const utils_1 = require("../utils");
const skeleton_1 = require("./skeleton");
class ImagesUploadProcess extends upload_1.FilesBatchUploadProcess {
    constructor(view, files, uploadHandler, position, opts) {
        super(view, files, uploadHandler);
        this.initPosition = position;
        this.createImage = (0, utils_1.createImageNode)((0, markdown_1.imageType)(this.view.state.schema), opts);
    }
    async createSkeleton() {
        return new skeleton_1.ImageSkeletonDescriptor(this.initPosition, await getSkeletonSize(this.files));
    }
    async createPMNode(res) {
        return this.createImage(res);
    }
}
exports.ImagesUploadProcess = ImagesUploadProcess;
async function getSkeletonSize(files) {
    const skeletonSize = { width: '300', height: '200' };
    if (files.length === 1) {
        try {
            const size = await (0, utils_1.loadImage)(files[0]).then(calcSkeletonSize);
            skeletonSize.width = String(size.width);
            skeletonSize.height = String(size.height);
        }
        catch (err) {
            logger_1.logger.error(err);
        }
    }
    return skeletonSize;
}
function calcSkeletonSize(img) {
    const { width, height } = img;
    if (height <= const_1.IMG_MAX_HEIGHT)
        return { width, height };
    const ratio = const_1.IMG_MAX_HEIGHT / height; // ratio<1
    return { height: const_1.IMG_MAX_HEIGHT, width: width * ratio };
}
