"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveCursorToEndOfMathInline = exports.removeEmptyMathInlineIfCursorIsAtBeginning = exports.ignoreIfCursorInsideMathInline = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const selection_1 = require("../../../utils/selection");
const const_1 = require("./const");
const ignoreIfCursorInsideMathInline = ({ selection, schema }) => {
    const $cursor = (0, selection_1.get$Cursor)(selection);
    return ($cursor === null || $cursor === void 0 ? void 0 : $cursor.parent.type) === (0, const_1.mathIType)(schema);
};
exports.ignoreIfCursorInsideMathInline = ignoreIfCursorInsideMathInline;
/**
 * If cursor at the beginning of MathInline:
 * - if math is empty – remove anchor
 * - if math has content – select node with NodeSelection
 */
const removeEmptyMathInlineIfCursorIsAtBeginning = ({ tr, schema }, dispatch) => {
    const $cursor = (0, selection_1.get$Cursor)(tr.selection);
    if (($cursor === null || $cursor === void 0 ? void 0 : $cursor.parent.type) === (0, const_1.mathIType)(schema) && $cursor.start() === $cursor.pos) {
        const isEmptyAnchor = $cursor.parent.nodeSize <= 2;
        if (isEmptyAnchor) {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.delete($cursor.before(), $cursor.after()));
        }
        else {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.setSelection(prosemirror_state_1.NodeSelection.create(tr.doc, $cursor.before())));
        }
        return true;
    }
    return false;
};
exports.removeEmptyMathInlineIfCursorIsAtBeginning = removeEmptyMathInlineIfCursorIsAtBeginning;
/**
 * If MathInline is before cursor – move cursor to the end of the math
 */
const moveCursorToEndOfMathInline = ({ tr, schema }, dispatch) => {
    var _a;
    const $cursor = (0, selection_1.get$Cursor)(tr.selection);
    if (((_a = $cursor === null || $cursor === void 0 ? void 0 : $cursor.nodeBefore) === null || _a === void 0 ? void 0 : _a.type) === (0, const_1.mathIType)(schema)) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, $cursor.pos - 1)));
        return true;
    }
    return false;
};
exports.moveCursorToEndOfMathInline = moveCursorToEndOfMathInline;
