"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmFilePaste = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_transform_1 = require("prosemirror-transform");
const lodash_1 = require("../../../../lodash");
const Clipboard_1 = require("../../../behavior/Clipboard");
const markdown_1 = require("../../../markdown");
const utils_1 = require("../../ImgSize/utils"); // TODO: remove hard import
const YfmFileSpecs_1 = require("../YfmFileSpecs");
const upload_1 = require("./upload");
const { isFilesOnly, isFilesFromHtml, isImageFile } = Clipboard_1.clipboardUtils;
const YfmFilePaste = (builder, opts) => {
    if (!opts || !(0, lodash_1.isFunction)(opts.fileUploadHandler))
        throw new Error('YfmFilePaste extension: fileUploadHandler is not a function');
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    const files = getPastedFiles(e.clipboardData);
                    if (files) {
                        e.preventDefault();
                        new YfmFilesPasteUploadProcess(view, files, {
                            uploadHandler: opts.fileUploadHandler,
                            needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages,
                        }).run();
                        return true;
                    }
                    return false;
                },
                drop(view, e) {
                    var _a, _b;
                    // handle drop files from device
                    if (view.dragging)
                        return false;
                    const files = getPastedFiles(e.dataTransfer);
                    if (!files)
                        return false;
                    const dropPos = (_b = (_a = view.posAtCoords({ left: e.clientX, top: e.clientY })) === null || _a === void 0 ? void 0 : _a.pos) !== null && _b !== void 0 ? _b : -1;
                    if (dropPos === -1)
                        return false;
                    const posToInsert = (0, prosemirror_transform_1.dropPoint)(view.state.doc, dropPos, createFakeFileSlice(view.state.schema));
                    if (posToInsert !== null) {
                        new YfmFilesPasteUploadProcess(view, files, {
                            pos: posToInsert,
                            uploadHandler: opts.fileUploadHandler,
                            needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages,
                        }).run();
                    }
                    e.preventDefault();
                    return true;
                },
            },
        },
    }), builder.Priority.Low);
};
exports.YfmFilePaste = YfmFilePaste;
function getPastedFiles(data) {
    if (!data)
        return null;
    if (!isFilesOnly(data) && !isFilesFromHtml(data))
        return null;
    return Array.from(data.files);
}
function createFakeFileSlice(schema) {
    return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from((0, YfmFileSpecs_1.fileType)(schema).create({
        href: 'fake',
        download: 'file',
    })), 0, 0);
}
class YfmFilesPasteUploadProcess extends upload_1.YfmFilesUploadProcessBase {
    constructor(view, files, opts) {
        super(view, files, opts.uploadHandler);
        this.pos = opts.pos;
        const { schema } = this.view.state;
        if ((0, markdown_1.imageType)(schema)) {
            this.createImage = (0, utils_1.createImageNode)((0, markdown_1.imageType)(schema), {
                needDimmensions: opts.needToSetDimensionsForUploadedImages,
            });
        }
    }
    getSkeletonInitPos() {
        var _a;
        return (_a = this.pos) !== null && _a !== void 0 ? _a : this.view.state.tr.selection.from;
    }
    async createPMNode(res) {
        return isImageFile(res.file) && this.createImage
            ? this.createImage(res)
            : this.createFile(res);
    }
}
