"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.insertFiles = exports.insertImages = exports.insertAnchor = exports.insertLink = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const defaultLinkSnippet = (0, autocomplete_1.snippet)(`[#{2:link}](#{1:url} "#{3:title}")`);
const insertLink = ({ state, dispatch }) => {
    const { from, to, empty } = state.selection.main;
    const linkSnippet = empty
        ? defaultLinkSnippet
        : (0, autocomplete_1.snippet)(`[#{2:${state.sliceDoc(from, to)}}](#{1:url} "#{3:title}")`);
    linkSnippet({ state, dispatch }, null, from, to);
    return true;
};
exports.insertLink = insertLink;
const defaultAnchorSnippet = (0, autocomplete_1.snippet)(`#[#{2:text}](#{1:anchor} "#{3:title}")`);
const insertAnchor = ({ state, dispatch }) => {
    const { from, to, empty } = state.selection.main;
    const anchorSnippet = empty
        ? defaultAnchorSnippet
        : (0, autocomplete_1.snippet)(`#[#{2:${state.sliceDoc(from, to)}}](#{1:anchor} "#{3:title}")`);
    anchorSnippet({ state, dispatch }, null, from, to);
    return true;
};
exports.insertAnchor = insertAnchor;
function insertImages(images) {
    return ({ state, dispatch }) => {
        const markup = images
            .map(({ title, url, alt, width, height }) => {
            const titleStr = title ? ` "${title}"` : '';
            const sizeStr = (width !== null && width !== void 0 ? width : height) ? ` =${width !== null && width !== void 0 ? width : ''}x${height !== null && height !== void 0 ? height : ''}` : '';
            return `![${alt !== null && alt !== void 0 ? alt : ''}](${url !== null && url !== void 0 ? url : ''}${titleStr}${sizeStr})`;
        })
            .join(' ');
        const tr = state.changeByRange((range) => {
            const changes = state.changes({ from: range.from, to: range.to, insert: markup });
            return { changes, range: range.map(changes) };
        });
        dispatch(state.update(tr));
        return true;
    };
}
exports.insertImages = insertImages;
const insertFiles = (files) => {
    return ({ state, dispatch }) => {
        const markup = files
            .map((attrs) => {
            const attrsStr = Object.entries(attrs)
                .map(([key, value]) => `${key}="${value.replace('"', '')}"`)
                .join(' ');
            return `{% file ${attrsStr} %}`;
        })
            .join(' ');
        const tr = state.changeByRange((range) => {
            const changes = state.changes({ from: range.from, to: range.to, insert: markup });
            return { changes, range: range.map(changes) };
        });
        dispatch(state.update(tr));
        return true;
    };
};
exports.insertFiles = insertFiles;
