import React, { useEffect, useImperativeHandle, useMemo, useReducer, useRef } from 'react';
import { Eye } from '@gravity-ui/icons';
import { Label } from '@gravity-ui/uikit';
import { cn } from '../classname';
import { i18n } from '../i18n/bundle';
import { throttle } from '../lodash';
import { getEditorLineNumberForOffset, scrollToRevealSourceLine, } from '../utils/sync-scroll';
const b = cn('markup-preview');
const SplitModeView = React.forwardRef(({ editor }, ref) => {
    var _a;
    const cm = editor.cm;
    const [, forceUpdate] = useReducer((x) => x + 1, 0);
    const outerRef = useRef(null);
    const lineElements = useRef([]);
    const updateLineElements = () => {
        if (!outerRef.current)
            return;
        const els = Array.from(outerRef.current.querySelectorAll('.line'));
        lineElements.current = els
            .filter((el) => { var _a; return el instanceof HTMLElement && ((_a = el.parentElement) === null || _a === void 0 ? void 0 : _a.offsetHeight) !== 0; })
            .map((el) => ({
            el: el,
            line: Number(el.getAttribute('data-line')),
        }));
    };
    const handleEditorScroll = useMemo(() => throttle(() => {
        if (!outerRef.current)
            return;
        if (!cm.dom.matches(':hover'))
            return;
        const { range } = cm.scrollSnapshot().value;
        const line = cm.state.doc.lineAt(range.from).number - 1;
        updateLineElements();
        scrollToRevealSourceLine(line, lineElements.current, outerRef);
    }, 30), [cm]);
    const handlePreviewScroll = useMemo(() => throttle(() => {
        if (!outerRef.current)
            return;
        if (!outerRef.current.matches(':hover'))
            return;
        updateLineElements();
        let lineNumber = getEditorLineNumberForOffset(outerRef.current.scrollTop, lineElements.current, outerRef);
        if (lineNumber === null || isNaN(lineNumber))
            return;
        lineNumber = Math.max(Math.floor(lineNumber), 0) + 1;
        const line = cm.state.doc.line(lineNumber);
        const { top } = cm.lineBlockAt(line.from);
        cm.scrollDOM.scrollTo({ top });
    }, 30), [cm]);
    useEffect(() => {
        const outer = outerRef.current;
        editor.on('change', forceUpdate);
        editor.on('cm-scroll', handleEditorScroll);
        outer === null || outer === void 0 ? void 0 : outer.addEventListener('scroll', handlePreviewScroll);
        editor.cm.requestMeasure();
        return () => {
            editor.off('change', forceUpdate);
            editor.off('cm-scroll', handleEditorScroll);
            outer === null || outer === void 0 ? void 0 : outer.removeEventListener('scroll', handlePreviewScroll);
        };
    }, [editor, handleEditorScroll, handlePreviewScroll]);
    useImperativeHandle(ref, () => outerRef.current);
    return (React.createElement("div", { className: b('outer', {
            vertical: editor.splitMode === 'vertical',
        }), ref: outerRef },
        React.createElement(Label, { icon: React.createElement(Eye, null), className: b('preview-sign'), size: 'm' }, i18n('split-mode-text')), (_a = editor.renderPreview) === null || _a === void 0 ? void 0 :
        _a.call(editor, { getValue: editor.getValue, mode: 'split' })));
});
SplitModeView.displayName = 'SplitModeView';
export { SplitModeView };
