var _WysiwygEditor_view, _WysiwygEditor_serializer, _WysiwygEditor_parser, _WysiwygEditor_actions, _WysiwygEditor_contentHandler, _WysiwygEditor_escapeConfig;
import { __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
import { EditorState } from 'prosemirror-state';
import { EditorView } from 'prosemirror-view';
import { WysiwygContentHandler } from './ContentHandler';
import { ExtensionsManager } from './ExtensionsManager';
import { bindActions } from './utils/actions';
import { logTransactionMetrics } from './utils/metrics';
export class WysiwygEditor {
    constructor({ domElem, initialContent = '', extensions = () => { }, allowHTML, mdPreset, linkify, linkifyTlds, escapeConfig, onChange, onDocChange, }) {
        _WysiwygEditor_view.set(this, void 0);
        _WysiwygEditor_serializer.set(this, void 0);
        _WysiwygEditor_parser.set(this, void 0);
        _WysiwygEditor_actions.set(this, void 0);
        _WysiwygEditor_contentHandler.set(this, void 0);
        _WysiwygEditor_escapeConfig.set(this, void 0);
        const { schema, markupParser: parser, serializer, nodeViews, markViews, plugins, rawActions, actions, } = ExtensionsManager.process(extensions, {
            // "breaks" option only affects the renderer, but not the parser
            mdOpts: { html: allowHTML, linkify, breaks: true, preset: mdPreset },
            linkifyTlds,
        });
        const state = EditorState.create({
            schema,
            doc: parser.parse(initialContent),
            plugins,
        });
        const thisOnChange = () => this.tryOnChange(onChange);
        const thisOnDocChange = () => this.tryOnChange(onDocChange);
        __classPrivateFieldSet(this, _WysiwygEditor_view, new EditorView(domElem !== null && domElem !== void 0 ? domElem : null, {
            state,
            nodeViews,
            markViews,
            dispatchTransaction(tr) {
                const newState = this.state.apply(tr);
                // @ts-expect-error
                this.updateState(newState);
                thisOnChange();
                if (tr.docChanged) {
                    thisOnDocChange();
                }
                logTransactionMetrics(tr);
            },
        }), "f");
        __classPrivateFieldSet(this, _WysiwygEditor_actions, actions.setActions(bindActions(rawActions)(__classPrivateFieldGet(this, _WysiwygEditor_view, "f"))), "f");
        __classPrivateFieldSet(this, _WysiwygEditor_serializer, serializer, "f");
        __classPrivateFieldSet(this, _WysiwygEditor_parser, parser, "f");
        __classPrivateFieldSet(this, _WysiwygEditor_contentHandler, new WysiwygContentHandler(__classPrivateFieldGet(this, _WysiwygEditor_view, "f"), parser), "f");
        __classPrivateFieldSet(this, _WysiwygEditor_escapeConfig, escapeConfig, "f");
    }
    get dom() {
        return __classPrivateFieldGet(this, _WysiwygEditor_view, "f").dom;
    }
    get serializer() {
        return __classPrivateFieldGet(this, _WysiwygEditor_serializer, "f");
    }
    get parser() {
        return __classPrivateFieldGet(this, _WysiwygEditor_parser, "f");
    }
    get actions() {
        return __classPrivateFieldGet(this, _WysiwygEditor_actions, "f").actions;
    }
    /** @internal used for prosemirror-dev-tools in demo */
    get view() {
        return __classPrivateFieldGet(this, _WysiwygEditor_view, "f");
    }
    action(actionName) {
        return __classPrivateFieldGet(this, _WysiwygEditor_actions, "f").action(actionName);
    }
    focus() {
        return __classPrivateFieldGet(this, _WysiwygEditor_view, "f").focus();
    }
    hasFocus() {
        return __classPrivateFieldGet(this, _WysiwygEditor_view, "f").hasFocus();
    }
    getValue() {
        return __classPrivateFieldGet(this, _WysiwygEditor_serializer, "f").serialize(__classPrivateFieldGet(this, _WysiwygEditor_view, "f").state.doc, __classPrivateFieldGet(this, _WysiwygEditor_escapeConfig, "f"));
    }
    isEmpty() {
        var _a;
        const { doc } = __classPrivateFieldGet(this, _WysiwygEditor_view, "f").state;
        return (doc.childCount === 1 &&
            ((_a = doc.firstChild) === null || _a === void 0 ? void 0 : _a.type.name) === 'paragraph' &&
            doc.firstChild.childCount === 0);
    }
    clear() {
        return __classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").clear();
    }
    replace(newMarkup) {
        return __classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").replace(newMarkup);
    }
    prepend(markup) {
        return __classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").prepend(markup);
    }
    append(markup) {
        return __classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").append(markup);
    }
    moveCursor(position) {
        return __classPrivateFieldGet(this, _WysiwygEditor_contentHandler, "f").moveCursor(position);
    }
    destroy() {
        __classPrivateFieldGet(this, _WysiwygEditor_view, "f").destroy();
    }
    tryOnChange(cb) {
        if (cb) {
            try {
                cb(this);
            }
            catch (err) {
                console.error(err);
            }
        }
    }
}
_WysiwygEditor_view = new WeakMap(), _WysiwygEditor_serializer = new WeakMap(), _WysiwygEditor_parser = new WeakMap(), _WysiwygEditor_actions = new WeakMap(), _WysiwygEditor_contentHandler = new WeakMap(), _WysiwygEditor_escapeConfig = new WeakMap();
