import React from 'react';
import { ActionTooltip, Button, DropdownMenu } from '@gravity-ui/uikit';
import { cn } from '../../../../classname';
import { i18n } from '../../../../i18n/gpt/dialog';
import { gptHotKeys } from '../constants';
import { useGptHotKeys } from '../hooks/useGptHotKeys';
import { usePresetList } from '../hooks/usePresetList';
import './Presetlist.css';
export const cnGptDialogPresetList = cn('gpt-dialog-preset-list');
const PresetItem = ({ preset, onPresetClick, disablePromptPresets, hotKey }) => {
    useGptHotKeys(hotKey, () => {
        onPresetClick(preset.data);
    }, { enableOnFormTags: true, enableOnContentEditable: true });
    return (React.createElement(ActionTooltip, { title: preset.display, hotkey: hotKey },
        React.createElement(Button, { className: cnGptDialogPresetList('preset'), view: "normal", size: "m", disabled: disablePromptPresets, onClick: () => onPresetClick(preset.data) }, preset.display)));
};
export const PresetList = ({ disablePromptPresets, promptPresets, onPresetClick, }) => {
    const { presetsContainerRef, visiblePresets, hiddenPresets, showMoreButton, measured } = usePresetList({
        promptPresets,
        onPresetClick,
    });
    return (React.createElement("div", { className: cnGptDialogPresetList({ measured }), ref: presetsContainerRef },
        visiblePresets.map((preset, index) => (React.createElement(PresetItem, { preset: preset, key: preset.display, hotKey: preset.hotKey || gptHotKeys.presetsKey(String(index + 1)), disablePromptPresets: disablePromptPresets, onPresetClick: onPresetClick }))),
        showMoreButton && (React.createElement(DropdownMenu, { switcherWrapperClassName: cnGptDialogPresetList('more-button-wrapper'), switcher: React.createElement(Button, { className: cnGptDialogPresetList('more-button'), view: "normal", size: "m" }, i18n('more')), items: hiddenPresets }))));
};
