import { useLayoutEffect, useMemo, useState } from 'react';
import debounceFn from 'lodash/debounce';
export function useOverflowingHorizontalItems({ containerRef, items, itemSelector, moreButtonSelector, marginBetweenItems = 0, }) {
    const [containerWidth, setContainerWidth] = useState(0);
    const [itemWidths, setItemWidths] = useState([]);
    const [moreButtonWidth, setMoreButtonWidth] = useState(0);
    useLayoutEffect(() => {
        const measureItemSizes = () => {
            var _a, _b, _c;
            const itemElements = Array.from((_b = (_a = containerRef.current) === null || _a === void 0 ? void 0 : _a.querySelectorAll(itemSelector)) !== null && _b !== void 0 ? _b : []);
            const moreButton = (_c = containerRef.current) === null || _c === void 0 ? void 0 : _c.querySelector(moreButtonSelector);
            setItemWidths(itemElements.map((item) => item.clientWidth));
            if (moreButton) {
                setMoreButtonWidth(moreButton.clientWidth);
            }
        };
        requestAnimationFrame(measureItemSizes);
    }, [containerRef, itemSelector, moreButtonSelector]);
    useLayoutEffect(() => {
        const container = containerRef.current;
        if (!container) {
            return;
        }
        const updateContainerSize = (entries) => {
            if (entries.length > 0) {
                setContainerWidth(entries[0].contentRect.width);
            }
        };
        const updateContainerSizeDebounced = debounceFn(updateContainerSize, 100);
        const containerResizeObserver = new ResizeObserver(updateContainerSizeDebounced);
        containerResizeObserver.observe(container);
        return () => containerResizeObserver.unobserve(container);
    }, [containerRef]);
    const isMeasured = itemWidths.length > 0;
    const { visibleItems, hiddenItems } = useMemo(() => {
        var _a, _b;
        if (!isMeasured) {
            return {
                visibleItems: items !== null && items !== void 0 ? items : [],
                hiddenItems: [],
            };
        }
        const itemsCount = itemWidths.length;
        let visibleItemsCount = 0;
        const spaceForMoreButton = moreButtonWidth + marginBetweenItems;
        let remainingContainerWidth = containerWidth;
        for (const width of itemWidths) {
            const itemWidthWithMargin = width + marginBetweenItems;
            remainingContainerWidth -= itemWidthWithMargin;
            if (remainingContainerWidth < spaceForMoreButton) {
                const isMoreThanOneItemLeft = itemsCount !== visibleItemsCount + 1;
                const hasNoSpaceForTheLastItem = remainingContainerWidth < 0;
                if (isMoreThanOneItemLeft || hasNoSpaceForTheLastItem) {
                    break;
                }
            }
            visibleItemsCount++;
        }
        return {
            visibleItems: (_a = items === null || items === void 0 ? void 0 : items.slice(0, visibleItemsCount)) !== null && _a !== void 0 ? _a : [],
            hiddenItems: (_b = items === null || items === void 0 ? void 0 : items.slice(visibleItemsCount)) !== null && _b !== void 0 ? _b : [],
        };
    }, [containerWidth, isMeasured, itemWidths, items, marginBetweenItems, moreButtonWidth]);
    return { visibleItems, hiddenItems, measured: isMeasured };
}
