import React from 'react';
import { Picture as ImageIcon } from '@gravity-ui/icons';
import { Icon, Popup } from '@gravity-ui/uikit';
import { useMountedState } from 'react-use';
import { cn } from '../../../../classname';
import { ImageForm } from '../../../../forms/ImageForm';
import { i18n } from '../../../../i18n/widgets';
import { useBooleanState } from '../../../../react-utils/hooks';
import './view.css';
const b = cn('image-placeholder');
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
export const FilePlaceholder = ({ onCancel, onSubmit, onAttach }) => {
    const isMounted = useMountedState();
    const [loading, showLoading, hideLoading] = useBooleanState(false);
    const divRef = React.useRef(null);
    const attachHandler = React.useCallback((files) => {
        if (!onAttach)
            return;
        if (isMounted()) {
            showLoading();
            onAttach(files).finally(() => {
                if (isMounted()) {
                    hideLoading();
                }
            });
        }
    }, [isMounted, onAttach, showLoading, hideLoading]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: divRef, className: b() },
            React.createElement(Icon, { data: ImageIcon, size: 24 }),
            i18n('image')),
        React.createElement(Popup, { open: true, onClose: onCancel, anchorRef: divRef, placement: placement },
            React.createElement(ImageForm, { autoFocus: true, loading: loading, onCancel: onCancel, onSubmit: onSubmit, onAttach: onAttach && attachHandler }))));
};
