import { NodeSelection, TextSelection } from 'prosemirror-state';
import { get$Cursor } from '../../../utils/selection';
import { mathIType } from './const';
export const ignoreIfCursorInsideMathInline = ({ selection, schema }) => {
    const $cursor = get$Cursor(selection);
    return ($cursor === null || $cursor === void 0 ? void 0 : $cursor.parent.type) === mathIType(schema);
};
/**
 * If cursor at the beginning of MathInline:
 * - if math is empty – remove anchor
 * - if math has content – select node with NodeSelection
 */
export const removeEmptyMathInlineIfCursorIsAtBeginning = ({ tr, schema }, dispatch) => {
    const $cursor = get$Cursor(tr.selection);
    if (($cursor === null || $cursor === void 0 ? void 0 : $cursor.parent.type) === mathIType(schema) && $cursor.start() === $cursor.pos) {
        const isEmptyAnchor = $cursor.parent.nodeSize <= 2;
        if (isEmptyAnchor) {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.delete($cursor.before(), $cursor.after()));
        }
        else {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.setSelection(NodeSelection.create(tr.doc, $cursor.before())));
        }
        return true;
    }
    return false;
};
/**
 * If MathInline is before cursor – move cursor to the end of the math
 */
export const moveCursorToEndOfMathInline = ({ tr, schema }, dispatch) => {
    var _a;
    const $cursor = get$Cursor(tr.selection);
    if (((_a = $cursor === null || $cursor === void 0 ? void 0 : $cursor.nodeBefore) === null || _a === void 0 ? void 0 : _a.type) === mathIType(schema)) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr.setSelection(TextSelection.create(tr.doc, $cursor.pos - 1)));
        return true;
    }
    return false;
};
