import React from 'react';
import { createPortal } from 'react-dom';
import { getReactRendererFromState } from '../../../behavior';
import { MermaidConsts } from '../MermaidSpecs/const';
import { MermaidView } from './MermaidView';
let mermaidInstance;
export class WMermaidNodeView {
    constructor(node, view, getPos, opts) {
        this.getMermaidInstance = () => mermaidInstance;
        const { loadRuntimeScript } = opts;
        this.node = node;
        this.dom = document.createElement('div');
        this.dom.classList.add('mermaid-container');
        this.dom.contentEditable = 'false';
        this.view = view;
        this.getPos = getPos;
        this.loadRuntimeScript = loadRuntimeScript;
        this.initializeMermaid();
        this.renderItem = getReactRendererFromState(view.state).createItem('mermaid-view', this.renderMermaid.bind(this));
    }
    initializeMermaid() {
        // https://github.com/diplodoc-platform/mermaid-extension/tree/master#prepared-mermaid-runtime
        window.mermaidJsonp = window.mermaidJsonp || [];
        window.mermaidJsonp.push((mermaid) => {
            mermaidInstance = mermaid;
        });
        this.loadRuntimeScript();
    }
    update(node) {
        if (node.type !== this.node.type)
            return false;
        this.node = node;
        this.renderItem.rerender();
        return true;
    }
    destroy() {
        this.renderItem.remove();
    }
    ignoreMutation() {
        return true;
    }
    stopEvent(e) {
        const target = e.target;
        if (typeof target.className === 'string' &&
            target.className.includes('prosemirror-stop-event')) {
            return true;
        }
        return false;
    }
    onChange(attrs) {
        const pos = this.getPos();
        if (pos === undefined)
            return;
        const tr = this.view.state.tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, this.node.attrs), attrs), []);
        this.view.dispatch(tr);
    }
    renderMermaid() {
        return createPortal(React.createElement(MermaidView, { view: this.view, onChange: this.onChange.bind(this), node: this.node, getMermaidInstance: this.getMermaidInstance, getPos: this.getPos }), this.dom);
    }
}
