import type { RefObject } from 'react';
import type { HotkeyProps, IconProps } from '@gravity-ui/uikit';
import type { ClassNameProps } from '../classname';
import type { ToolbarListButtonData } from './ToolbarListButton';
export declare type ToolbarBaseProps<E> = ClassNameProps & {
    editor: E;
    focus(): void;
    onClick?(id: string, attrs?: {
        [key: string]: any;
    }): void;
};
export declare type ToolbarIconData = Pick<IconProps, 'data' | 'size'>;
export declare type ToolbarItemData<E> = {
    id: string;
    icon: ToolbarIconData;
    title: string | (() => string);
    hint?: string | (() => string);
    hotkey?: HotkeyProps['value'];
    /** @deprecated Use _hintWhenDisabled_ setting instead */
    disabledPopoverVisible?: boolean;
    /**
     * Show hint when _isEnable()_ returns false
     *
     * `false` – don't show hint;
     *
     * `true` – show default hint;
     *
     * `string` or `() => string` – show hint with custom message.
     *
     * @default true
     */
    hintWhenDisabled?: boolean | string | (() => string);
    exec(editor: E): void;
    isActive(editor: E): boolean;
    isEnable(editor: E): boolean;
};
export declare enum ToolbarDataType {
    SingleButton = "s-button",
    ListButton = "list-b",
    ButtonPopup = "b-popup",
    ReactNode = "r-node",
    ReactNodeFn = "r-node-fn",
    ReactComponent = "r-component"
}
export declare type ToolbarGroupItemData<E> = ToolbarSingleItemData<E> | ToolbarButtonPopupData<E> | ToolbarListItemData<E> | ToolbarReactNodeData | ToolbarReactNodeFnData<E> | ToolbarReactComponentData<E>;
export declare type ToolbarSingleItemData<E> = ToolbarItemData<E> & {
    id: string;
    type: ToolbarDataType.SingleButton;
    className?: string;
};
export declare type ToolbarListItemData<E> = ToolbarListButtonData<E> & {
    id: string;
    type: ToolbarDataType.ListButton;
    className?: string;
};
export declare type ToolbarReactComponentData<E> = {
    id: string;
    type: ToolbarDataType.ReactComponent;
    width: number;
    className?: string;
    component: React.ComponentType<ToolbarBaseProps<E>>;
};
export declare type ToolbarButtonPopupData<E> = ToolbarItemData<E> & {
    /** not used, may be an empty function */
    exec: ToolbarItemData<E>['exec'];
    type: ToolbarDataType.ButtonPopup;
    renderPopup: (props: ToolbarBaseProps<E> & {
        hide: () => void;
        anchorRef: RefObject<HTMLElement>;
    }) => React.ReactNode;
    className?: string;
};
export declare type ToolbarListButtonItemData<E> = ToolbarItemData<E> & {
    doNotActivateList?: boolean;
};
export declare type ToolbarReactNodeData = {
    id: string;
    type: ToolbarDataType.ReactNode;
    width: number;
    content: React.ReactNode;
};
export declare type ToolbarReactNodeFnData<E> = {
    id: string;
    type: ToolbarDataType.ReactNodeFn;
    width: number;
    content: (e: E) => React.ReactNode;
};
